//*******************************************************************************
//
//  File name :       dm_gen.v
//
//  Description :     This module contains logic to generate the data mask
//                    for a DDR-DIMM
//
// 
//                    
//  Date - revision : 6/20/2004
//
//  Author :          Kraig Lund
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************


`define idle_dm                  4'h0//0_0000
`define first_dm                 4'h1//0_0001
`define second_dm                4'h2//0_0010      
`define middle_burst_dm          4'h3//0_0011
`define second_to_last_dm        4'h4//0_0100 
`define last_dm                  4'h5//0_0101


module dm_gen(
              clk,
                rst,
              addr_fifo,
              cmd_fifo,
              wrburst_cnt,
              burst_n_dword,
              first_write,
              ctrl_Dm_En,
              pci_be
             );

  input        clk; 
  input        rst;
  input [26:0] addr_fifo;
  input [23:0] cmd_fifo;
  input [8:0]  wrburst_cnt;
  input        burst_n_dword;
  input        first_write;
  output[15:0] ctrl_Dm_En;
  input [7:0]  pci_be;

reg [15:0] ctrl_Dm_En_r, ctrl_Dm_En_r1;
reg [3:0]  state_dm;
reg [13:0] first_addr, last_addr;
wire[11:0] byte_count;
wire[4:2]  byte_plus_addr;
reg [2:0]  dm_sel;

assign ctrl_Dm_En = (burst_n_dword == 1'b0) ? (ctrl_Dm_En_r1 | {2{pci_be[3:0],pci_be[7:4]}}) : ctrl_Dm_En_r1;  

//Data Mask (DM) Generation
wire [31:0] first_dm_beat;
wire [31:0] last_dm_beat;

assign byte_plus_addr[4:2] = addr_fifo[4:2] + byte_count[4:2];
assign byte_count = cmd_fifo[19:8];

always@(byte_count[11:0]or addr_fifo[4:2] or byte_plus_addr[4:2])begin
  if(byte_count >= 0 && byte_count < 8'h24) begin//less than 36 bytes
     first_addr[13:0] =  {1'b0, 2'b00, byte_count[4:2], addr_fifo[4:2], 5'b00000};
     last_addr[13:0]  =  {1'b1, 2'b00, byte_count[4:2], addr_fifo[4:2], 5'b00000};
  end else begin
       first_addr[13:0] =  {1'b0, 1'b0, 4'b1001, addr_fifo[4:2], 5'b00000};
     last_addr[13:0]  =  {1'b1, 1'b0, 4'b1001, byte_plus_addr[4:2], 5'b00000};
  end
end


//State machine to drive DM Mux 
  always @ (posedge clk) begin
  if (rst) begin
      state_dm <= `idle_dm;
      dm_sel <= 3'b000;
  end else begin
      case (state_dm)
      `idle_dm : begin
            dm_sel <= 3'b000;
            if(first_write == 1'b1)
               state_dm <= `first_dm;
            else
               state_dm <= `idle_dm;           
       end
      `first_dm : begin
            dm_sel <= 3'b010;
            state_dm <= `second_dm;
       end
      `second_dm : begin
            dm_sel <= 3'b011;                   
            if(burst_n_dword == 0)
               state_dm <= `idle_dm;
            else if (wrburst_cnt > 3)
               state_dm <= `middle_burst_dm;
            else
               state_dm <= `second_to_last_dm;
        end
       `middle_burst_dm: begin
            dm_sel <= 3'b001;
            if(wrburst_cnt > 3)
               state_dm <= `middle_burst_dm;
            else
               state_dm <= `second_to_last_dm;
         end
        `second_to_last_dm : begin
            dm_sel <= 3'b100;
            state_dm <= `last_dm;
         end
        `last_dm : begin
            dm_sel <= 3'b101;
            state_dm <= `idle_dm;
         end
      endcase
     end
    end         

  //DM mux

      always@(dm_sel or first_dm_beat or last_dm_beat) begin
      case(dm_sel)
         3'b000: begin
             ctrl_Dm_En_r[15:0] = 16'hffff;
         end
         3'b001: begin
             ctrl_Dm_En_r[15:0] = 16'h0000;
         end
         3'b010: begin
             ctrl_Dm_En_r[15:0] = first_dm_beat[31:16]; 
         end
         3'b011: begin
             ctrl_Dm_En_r[15:0] = first_dm_beat[15:0]; 
         end
         3'b100: begin
             ctrl_Dm_En_r[15:0] = last_dm_beat[31:16]; 
         end
         3'b101: begin
             ctrl_Dm_En_r[15:0] = last_dm_beat[15:0]; 
         end
         default : begin
             ctrl_Dm_En_r[15:0] = 16'hffff; end
        endcase
      end

      always@(posedge clk)begin
         ctrl_Dm_En_r1 <= ctrl_Dm_En_r;
      end


   RAMB16 #(
      .DOA_REG(0),  // Optional output registers on A port (0 or 1)
      .DOB_REG(0),  // Optional output registers on B port (0 or 1)
      .INIT_A(36'hFFFFFFFFF),  // Initial values on A output port
      .INIT_B(36'hFFFFFFFFF),  // Initial values on B output port
      .INVERT_CLK_DOA_REG("FALSE"),  // Invert clock on A port output registers ("TRUE" or "FALSE")
      .INVERT_CLK_DOB_REG("FALSE"),  // Invert clock on A port output registers ("TRUE" or "FALSE")
      .RAM_EXTENSION_A("NONE"),  // "UPPER", "LOWER" or "NONE" when cascaded
      .RAM_EXTENSION_B("NONE"),  // "UPPER", "LOWER" or "NONE" when cascaded
      .READ_WIDTH_A(36),  // Valid values are 1, 2, 4, 9, 18, or 36
      .READ_WIDTH_B(36),  // Valid values are 1, 2, 4, 9, 18, or 36
      .SIM_COLLISION_CHECK("ALL"),  // Collision check enable "ALL", "WARNING_ONLY", 
                                    //   "GENERATE_X_ONLY" or "NONE" 
      .SRVAL_A(36'hFFFFFFFFF), // Set/Reset value for A port output
      .SRVAL_B(36'hFFFFFFFFF),  // Set/Reset value for B port output
      .WRITE_MODE_A("WRITE_FIRST"),  // "WRITE_FIRST", "READ_FIRST", or "NO_CHANGE" 
      .WRITE_MODE_B("WRITE_FIRST"),  // "WRITE_FIRST", "READ_FIRST", or "NO_CHANGE" 
      .WRITE_WIDTH_A(36),  // Valid values are 1, 2, 4, 9, 18, or 36
      .WRITE_WIDTH_B(36),  // Valid values are 1, 2, 4, 9, 18, or 36

      // The following INIT_xx declarations specify the initial contents of the RAM
      //first_dm_beats
      .INIT_00(256'h00000000000000000000000000000000000000000000000000000000_ffffffff),
      .INIT_01(256'hfffffff0_ffffff0f_fffff0ff_ffff0fff_fff0ffff_ff0fffff_f0ffffff_0fffffff),//4-byte
      .INIT_02(256'hfffffff0_ffffff00_fffff00f_ffff00ff_fff00fff_ff00ffff_f00fffff_00ffffff),//8-byte
      .INIT_03(256'hfffffff0_ffffff00_fffff000_ffff000f_fff000ff_ff000fff_f000ffff_000fffff),//12-byte
      .INIT_04(256'hfffffff0_ffffff00_fffff000_ffff0000_fff0000f_ff0000ff_f0000fff_0000ffff),//16-byte
      .INIT_05(256'hfffffff0_ffffff00_fffff000_ffff0000_fff00000_ff00000f_f00000ff_00000fff),//20-byte
      .INIT_06(256'hfffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f000000f_000000ff),//24-byte
      .INIT_07(256'hfffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f0000000_0000000f),//28-byte
      .INIT_08(256'hfffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f0000000_00000000),//32-byte
      .INIT_09(256'hfffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f0000000_00000000),//36 and beyond
                                                                
      .INIT_0A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_0F(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_10(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_11(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_12(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_13(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_14(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_15(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_16(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_17(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_18(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_19(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_1F(256'h0000000000000000000000000000000000000000000000000000000000000000),

       //last_dm_beat
      .INIT_20(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_21(256'hffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff),//4-byte
      .INIT_22(256'h0fffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff),//8-byte
      .INIT_23(256'h00ffffff_0fffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff),//12-byte
      .INIT_24(256'h000fffff_00ffffff_0fffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff),//16-byte
      .INIT_25(256'h0000ffff_000fffff_00ffffff_0fffffff_ffffffff_ffffffff_ffffffff_ffffffff),//20-byte
      .INIT_26(256'h00000fff_0000ffff_000fffff_00ffffff_0fffffff_ffffffff_ffffffff_ffffffff),//24-byte
      .INIT_27(256'h000000ff_00000fff_0000ffff_000fffff_00ffffff_0fffffff_ffffffff_ffffffff),//28-byte
      .INIT_28(256'h0000000f_000000ff_00000fff_0000ffff_000fffff_00ffffff_0fffffff_ffffffff),//32-byte
      .INIT_29(256'h0000000f_000000ff_00000fff_0000ffff_000fffff_00ffffff_0fffffff_00000000),//36 and beyond
     
      .INIT_2A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_2F(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_30(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_31(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_32(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_33(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_34(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_35(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_36(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_37(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_38(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_39(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3A(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3B(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3C(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3D(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3E(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INIT_3F(256'h0000000000000000000000000000000000000000000000000000000000000000),
      
      // The next set of INITP_xx are for the parity bits
      .INITP_00(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_01(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_02(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_03(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_04(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_05(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_06(256'h0000000000000000000000000000000000000000000000000000000000000000),
      .INITP_07(256'h0000000000000000000000000000000000000000000000000000000000000000)
   ) 
   RAMB16_inst (
     // .CASCADEOUTA(CASCADEOUTA),  // 1-bit cascade output
     // .CASCADEOUTB(CASCADEOUTB),  // 1-bit cascade output
      .DOA(first_dm_beat),      // 32-bit A port data output
      .DOB(last_dm_beat),      // 32-bit B port data output
      .DOPA(),    // 4-bit A port parity data output
      .DOPB(),    // 4-bit B port parity data output
      .ADDRA({1'b0,first_addr}),  // 15-bit A port address input
      .ADDRB({1'b0,last_addr}),  // 15-bit B port address input
      .CASCADEINA(1'b0), // 1-bit cascade A input
      .CASCADEINB(1'b0), // 1-bit cascade B input
      .CLKA(clk),     // 1-bit A port clock input
      .CLKB(clk),     // 1-bit B port clock input
      .DIA(32'h00000000),       // 32-bit A port data input
      .DIB(32'h00000000),       // 32-bit B port data input
      .DIPA(4'b0000),     // 4-bit A port parity data input
      .DIPB(4'b0000),     // 4-bit B port parity data input
      .ENA(1'b1),       // 1-bit A port enable input
      .ENB(1'b1),       // 1-bit B port enable input
      .REGCEA(1'b0), // 1-bit A port register enable input
      .REGCEB(1'b0), // 1-bit B port register enable input
      .SSRA(rst),     // 1-bit A port set/reset input
      .SSRB(rst),     // 1-bit B port set/reset input
      .WEA(4'b0000),       // 4-bit A port write enable input
      .WEB(4'b0000)        // 4-bit B port write enable input
   );

   // End of RAMB16_inst instantiation
   endmodule
