//******************************************************************************
//
//  Xilinx, Inc. 2005                 www.xilinx.com
//
//
//*******************************************************************************
//
//  File name :       tap_ctrl.v
//
//  Description :     This module detects two transitions of a dqs (strobe) signal 
//                    and determines the tap delay required for the associated
//                    data bits in order to center them with respect to internal
//                    FPGA clock, CLK.
// 
//                    
//  Date - revision : 3/11/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

`define idelay_rst  3'b000
`define idle        3'b001
`define idelay_inc  3'b010
`define detect_edge 3'b011

module tap_ctrl(
                CLK,
                RESET,
                RDY_STATUS,
                DQS, 
                CTRL_DUMMYREAD_START,
                DLYINC, 
                DLYCE,  
                DLYRST,
                SEL_DONE,
                FIRST_RISING,
                VALID_DATA_TAP_COUNT,
                DATA_TAP_COUNT
                )/* synthesis syn_noprune=1 */;

   input        CLK;               
   input        RESET;
   input        RDY_STATUS;
   input        DQS;
   input        CTRL_DUMMYREAD_START;
                
   output       DLYINC;
   output       DLYCE;
   output       DLYRST;
   output       SEL_DONE;
   output       FIRST_RISING;
   output       VALID_DATA_TAP_COUNT;
   output [5:0] DATA_TAP_COUNT;
                             
   reg         prev_dqs_level;
   reg         dly_inc;
   reg         dly_ce;
   reg         dly_rst;
   reg [1:0]   transition;
   reg         rising;
   reg         rising_edge;
   reg         first_edge;
   reg         second_edge;
   reg         second_edge_r1;
   reg         second_edge_r2;
   reg         second_edge_r3;
   reg         transition_rst;
   reg         sel_complete;
   reg [5:0]   tap_counter;
   reg [5:0]   first_edge_tap_count;
   reg [5:0]   second_edge_tap_count;
   reg [5:0]   pulse_width_tap_count /* synthesis syn_noprune=1 */;
   reg [5:0]   data_bit_tap_count;
   reg [2:0]   state;
   reg         idelay_rst_idle;
   reg         idelay_rst_idle_r1;
   reg         idelay_inc_idle;
   reg         idelay_inc_idle_r1;
   reg         detect_edge_idle;
   reg         detect_edge_idle_r1;
   reg [3:0]   dly_after_first_cnt;
   reg [5:0]   pulse_center_tap_count /* synthesis syn_noprune=1 */;
   reg         valid_data_count;

   wire        data_count_valid;
   wire [3:0]  dly_after_first;
   wire        curr_dqs_level;
   wire        dqs_int;   
   wire        delay_sel_done;
   wire        reset_int;
   wire        gnd;

   wire [7:0]  unused_data_in;   
	
   
   assign DLYINC               = dly_inc;
   assign DLYCE                = dly_ce;
   assign DLYRST               = dly_rst;
   assign SEL_DONE             = sel_complete;
   assign FIRST_RISING         = rising_edge; 
   assign VALID_DATA_TAP_COUNT = valid_data_count;
   assign DATA_TAP_COUNT[5:0]  = data_bit_tap_count; 
     
   assign data_count_valid     = (second_edge_r3 || 
                                 (tap_counter[5:0] == 6'b111111)) 
                                 ? 1'b1 : 1'b0;
   assign reset_int            = (~RDY_STATUS) || RESET;
   assign gnd                  = 1'b0;
   assign unused_data_in[7:0]  = 8'b11111111;
  
   assign delay_sel_done = ((second_edge == 1'b1) || 
                            (tap_counter[5:0] == 6'b111111)) ? 1'b1 : 
                            (CTRL_DUMMYREAD_START == 1'b0)      ? 1'b0 :
                             sel_complete;
                           
   assign dly_after_first[3:0] = ((transition[1:0] == 2'b01) && (first_edge == 1'b0)) ? 4'b1001 :
                                 ((dly_after_first_cnt[3:0] != 4'b0000) && (dly_inc == 1'b1)) ? (dly_after_first_cnt[3:0] - 1'b1) :
                                 dly_after_first_cnt[3:0];                          
                           
// Two pipeline stages for DQS output from IDELAY block

FD dqs_r1 (
           .Q(dqs_int),
           .C(CLK),
           .D(DQS)
           )/* synthesis xc_props="IOB=TRUE"*/;
           
FD dqs_r2 (
           .Q(curr_dqs_level),
           .C(CLK),
           .D(dqs_int)
           )/* synthesis xc_props="IOB=TRUE"*/;           

// Shift registers for controls
   always @ (posedge CLK)
   begin
     if (reset_int == 1'b1)
       begin
         second_edge_r1      <= 1'b0;
         second_edge_r2      <= 1'b0;
         second_edge_r3      <= 1'b0;
         idelay_rst_idle_r1  <= 1'b0;
         idelay_inc_idle_r1  <= 1'b0;
         detect_edge_idle_r1 <= 1'b0;
         valid_data_count    <= 1'b0;
       end
     else
       begin
         second_edge_r1      <= second_edge;
         second_edge_r2      <= second_edge_r1;
         second_edge_r3      <= second_edge_r2;
         idelay_rst_idle_r1  <= idelay_rst_idle;
         idelay_inc_idle_r1  <= idelay_inc_idle;
         detect_edge_idle_r1 <= detect_edge_idle;
         valid_data_count    <= data_count_valid;
       end
   end

// Tap Delay Selection Complete for Data bus associated with a DQS                           
   always @ (posedge CLK)
   begin
     if (reset_int == 1'b1)
       sel_complete <= 1'b0;
     else
       sel_complete <= delay_sel_done;
   end
   
// Start detection of second transition only after 10 taps 
// from first transition
   always @ (posedge CLK)
   begin
     if (reset_int == 1'b1)
       dly_after_first_cnt[3:0] <= 4'b0000;
     else
       dly_after_first_cnt[3:0] <= dly_after_first[3:0];
   end   

// Tap Counter    
   always @ (posedge CLK)
   begin 
     if ((reset_int == 1'b1) || (tap_counter[5:0] == 6'b111111))
       tap_counter[5:0] <= 6'b000000;
     else if (dly_inc == 1'b1)
       tap_counter[5:0] <= tap_counter[5:0] + 1'b1;
   end


// Tap value for Data IDELAY circuit
   always @ (posedge CLK)
   begin 
     if (reset_int == 1'b1)
       first_edge_tap_count[5:0]   <= 6'b000000;
     else if ((transition[1:0] == 2'b01) && (first_edge == 1'b0))
       first_edge_tap_count[5:0]   <= tap_counter[5:0] - 2'b10;
   end
   
   always @ (posedge CLK)
   begin 
     if (reset_int == 1'b1)
       second_edge_tap_count[5:0]  <= 6'b000000;
     else if ((transition[1:0] == 2'b10) && (second_edge == 1'b0))
       second_edge_tap_count[5:0]  <= tap_counter[5:0] - 2'b10;
   end
       
   always @ (posedge CLK)
   begin 
     if (reset_int == 1'b1)
       pulse_width_tap_count[5:0] <= 6'b000000;
     else if (second_edge_r1 == 1'b1)
       pulse_width_tap_count[5:0] <= (second_edge_tap_count[5:0] - 
                                      first_edge_tap_count[5:0]);
   end
   
   always @ (posedge CLK)
   begin 
     if (reset_int == 1'b1)
       pulse_center_tap_count[5:0] <= 6'b000000;
     else if (second_edge_r2 == 1'b1)
       // Shift right to divide by 2 and find pulse center
       pulse_center_tap_count[5:0] <= pulse_width_tap_count[5:0] >> 1;  
   end   
       
   always @ (posedge CLK)
   begin 
     if (reset_int == 1'b1)
       data_bit_tap_count[5:0]     <= 6'b000000;
     else if (second_edge_r3 == 1'b1)  // 2 edges detected
       data_bit_tap_count[5:0]  <= first_edge_tap_count[5:0] + 
       pulse_center_tap_count[5:0];
     else if ((transition[1:0] == 2'b01) && ((tap_counter[5:0] == 6'b111111)))
       // Only 1 edge detected
       // The default is + 6'b010000;
       // For 133MHz DDR such as on the ML-455 a better number would be 25 taps
       // (0x19) or half the bit-rate
       // By subtracting instead of adding this value we get the best 
       // possible latency
       // For other designs it may not be possible to subtract depending 
       // on the amount of signal flight time
       // in which case it will be necesarry to add instead of subtract 
       data_bit_tap_count[5:0]  <= first_edge_tap_count[5:0] - 6'b011001;
     else if ((transition[1:0] == 2'b00) && ((tap_counter[5:0] == 6'b111111)))
       // No edges detected
       data_bit_tap_count[5:0]  <= 6'b100000;
   end                      


// First and second edge assignment logic 
   always @ (posedge CLK)
   begin
     if (reset_int == 1'b1)
       begin
         transition[1:0] <= 2'b00;
         rising          <= 1'b0;
       end
     else if ((dly_after_first_cnt == 4'b0000) && (state[2:0] == `detect_edge))
       begin        
         if ((curr_dqs_level != prev_dqs_level) && (transition_rst == 1'b0))
           begin 
             transition[1:0] <= transition[1:0] + 1'b1;
             if ((curr_dqs_level == 1'b0) && (prev_dqs_level == 1'b1))
               rising <= 1'b1;
             else
               rising <= 1'b0;
           end
       end
     else if (transition_rst == 1'b1)
       transition[1:0] <= 2'b00;
     else
       transition[1:0] <= transition[1:0];
   end
   
   always @ (posedge CLK)
   begin
     if (reset_int == 1'b1)
       begin
         transition_rst  <= 1'b0;
         first_edge      <= 1'b0;
         second_edge     <= 1'b0;
         rising_edge     <= 1'b0;
       end
     else
       begin   
         case (transition[1:0])
         2'b01: begin
                  first_edge <= 1'b1;
                  if (rising == 1'b1)
                    rising_edge <= 1'b1;
                  else
                    rising_edge <= 1'b0;
                end
         2'b10: begin
                  if (transition_rst == 1'b1)
                    begin
                      second_edge <= 1'b0;
                      transition_rst <= 1'b0;
                    end
                  else
                    begin
                      second_edge    <= 1'b1;
                      transition_rst <= 1'b1;
                    end
                end
         default: begin
                    first_edge     <= 1'b0; 
                    second_edge    <= 1'b0;
                  end
         endcase
       end
   end                     
     

// State Machine for edge detection and midpoint determination   
   always @ (posedge CLK)
   begin
     if (reset_int == 1'b1)   // DQS IDELAY in reset
       begin
         dly_rst           <= 1'b1;
         dly_ce            <= 1'b0;
         dly_inc           <= 1'b0;
         idelay_rst_idle   <= 1'b0;
         detect_edge_idle  <= 1'b0;
         idelay_inc_idle   <= 1'b0;
         prev_dqs_level    <= curr_dqs_level;
         state[2:0]        <= `idelay_rst;
       end
     else if ((CTRL_DUMMYREAD_START == 1'b1) && (sel_complete == 1'b0))
       begin
         case (state[2:0])
         3'b000: begin // idelay_rst 
                   dly_rst         <= 1'b1;
                   dly_ce          <= 1'b0;
                   dly_inc         <= 1'b0;
                   idelay_rst_idle <= 1'b1;
                   state[2:0]      <= `idle;
                 end
         3'b001: begin // idle 
                   dly_rst          <= 1'b0;
                   dly_ce           <= 1'b0;
                   dly_inc          <= 1'b0;
                   idelay_rst_idle  <= 1'b0;
                   detect_edge_idle <= 1'b0;
                   idelay_inc_idle  <= 1'b0;                   
                   if (idelay_rst_idle_r1 == 1'b1)
                     state[2:0] <= `idelay_inc;
                   else if ((idelay_inc_idle_r1 == 1'b1) || 
                           ((detect_edge_idle_r1 == 1'b1) && 
                           (second_edge_r1 == 1'b0) && 
                           (tap_counter[5:0] != 6'b111111)))
                     state[2:0] <= `detect_edge;
                   else 
                     state[2:0] <= `idle;
                 end
         3'b010: begin // idelay_inc 
                   dly_rst         <= 1'b0;
                   dly_ce          <= 1'b1;
                   dly_inc         <= 1'b1;
                   idelay_inc_idle <= 1'b1;
                   state[2:0]      <= `idle;
                   prev_dqs_level  <= curr_dqs_level;
                 end
         3'b011: begin // detect_edge 
                   dly_rst          <= 1'b0;
                   dly_ce           <= 1'b1;
                   dly_inc          <= 1'b1;
                   detect_edge_idle <= 1'b1;
                   state[2:0]       <= `idle;
                   prev_dqs_level   <= curr_dqs_level;
                 end
         default: begin
                    dly_rst          <= 1'b0;          
                    dly_ce           <= 1'b0;          
                    dly_inc          <= 1'b0;
                    idelay_rst_idle  <= 1'b0;
                    detect_edge_idle <= 1'b0;
                    idelay_inc_idle  <= 1'b0;          
                    prev_dqs_level   <= curr_dqs_level;
                    state[2:0]       <= `idle;   
                  end
         endcase
       end  
     else
       begin
         dly_rst          <= 1'b0;
         dly_ce           <= 1'b0;
         dly_inc          <= 1'b0;
         idelay_rst_idle  <= 1'b0;
         detect_edge_idle <= 1'b0;
         idelay_inc_idle  <= 1'b0;
         prev_dqs_level   <= curr_dqs_level;
         state[2:0]       <= `idelay_rst;
       end    
   end            
  

endmodule

