--*******************************************************************************
-- *******************************************************************************
--    
--  Xilinx, Inc. 2005                 www.xilinx.com
--    
--  Synthesizable DDR SDRAM Controller
--    
-- *********************************************************************************
--    
--  File name :   ddr_controller.vhd
--    
--  Description     : This module 
--                     1. provides the read enable signals to the Write Address,
--                        Write Data and the Read Address FIFOs
--                     2. has the controller state machine which supplies the
--                        right command signals to the ddr device. Auto Refresh 
--                        commands are generated by the controller taking the 
--                        Auto Refresh Command Interval into account. The controller
--                        also provides commands for enabling ODT and 
--                        OCD(Off chip driver calibration).
--                     3. provides the address signals to the ddr device.
--    
--    
--  Date - revision : 
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--    
--  Author :      Lakshmi Gopalakrishnan
--    
--  Contact :     e-mail  hotline@xilinx.com
--                phone   + 1 800 255 7778 
--    
--  Disclaimer:   LIMITED WARRANTY AND DISCLAMER. These designs are 
--                provided to you "as is". Xilinx and its licensors make and you 
--                receive no warranties or conditions, express, implied, 
--                statutory or otherwise, and Xilinx specifically disclaims any 
--                implied warranties of merchantability, non-infringement, or 
--                fitness for a particular purpose. Xilinx does not warrant that 
--                the functions contained in these designs will meet your 
--                requirements, or that the operation of these designs will be 
--                uninterrupted or error free, or that defects in the Designs 
--                will be corrected. Furthermore, Xilinx does not warrant or 
--                make any representations regarding use or the results of the 
--                use of the designs in terms of correctness, accuracy, 
--                reliability, or otherwise. 
--    
--                LIMITATION OF LIABILITY. In no event will Xilinx or its 
--                licensors be liable for any loss of data, lost profits, cost 
--                or procurement of substitute goods or services, or for any 
--                special, incidental, consequential, or indirect damages 
--                arising from the use or operation of the designs or 
--                accompanying documentation, however caused and on any theory 
--                of liability. This limitation will apply even if Xilinx 
--                has been advised of the possibility of such damage. This 
--                limitation shall apply not-withstanding the failure of the 
--                essential purpose of any limited remedies herein. 
--    
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
--    
-- *****************************************************************************
                                

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  USE IEEE.std_logic_unsigned.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY ddr_controller IS
   PORT (
      -- controller inputs
      clk266                  : IN std_logic;   
      refresh_clk             : IN std_logic;   
      rst                     : IN std_logic;   
      -- FIFO  signals
      addr_fifo               : IN std_logic_vector(26 DOWNTO 0);   
      cmd_fifo                : IN std_logic_vector(23 DOWNTO 0);   
      phy_Dly_Slct_Done       : IN std_logic;   
      addr_fifo_empty         : IN std_logic;   
      cmd_fifo_empty          : IN std_logic;   
      ctrl_Dummyread_Start    : OUT std_logic;   
      -- FIFO read enable signals
      ctrl_af_RdEn            : OUT std_logic;   
      ctrl_Wdf_RdEn           : OUT std_logic;   
      wdf_empty               : IN std_logic;   
      ctrl_cmd_RdEn           : OUT std_logic;   
      -- Rst and Enable signals for DQS logic
      ctrl_Dqs_Rst            : OUT std_logic;   
      ctrl_Dqs_En             : OUT std_logic;   
      ctrl_Dm_En              : OUT std_logic_vector(15 DOWNTO 0);   
      -- Read and Write Enable signals to the phy interface
      ctrl_WrEn               : OUT std_logic;   
      ctrl_RdEn               : OUT std_logic;   
      ctrl_init_done          : OUT std_logic;   
      ddr_address             : OUT std_logic_vector(11 DOWNTO 0);   
      ddr_ba                  : OUT std_logic_vector(1 DOWNTO 0);   
      ddr_ras_L               : OUT std_logic;   
      ddr_cas_L               : OUT std_logic;   
      ddr_we_L                : OUT std_logic;   
      ddr_cs_L                : OUT std_logic_vector(1 DOWNTO 0);   
      ddr_cke                 : OUT std_logic;   
      ddr_odt                 : OUT std_logic;   
      not_ready               : OUT std_logic;   
      burst_half              : IN std_logic;   
      PCI_HIT                 : IN std_logic;   
      PCI_HIT_EARLY           : IN std_logic;   
      PCI_BE                  : IN std_logic_vector(7 DOWNTO 0);   
      PCI_ADDR_EARLY          : IN std_logic_vector(26 DOWNTO 0);   
      PCI_CBEA_EARLY          : IN std_logic_vector(7 DOWNTO 0);   
      first_read              : OUT std_logic;   
      S_DONE                  : IN std_logic);   
END ddr_controller;

ARCHITECTURE rtl OF ddr_controller IS

 constant idle_ctrl: std_logic_vector(4 downto 0) := "00000";
 constant load_mode_reg : std_logic_vector(4 downto 0) := "00001";
 constant mode_register_wait : std_logic_vector(4 downto 0) := "00010";
 constant precharge : std_logic_vector(4 downto 0) := "00011";
 constant precharge_wait : std_logic_vector(4 downto 0) := "00100"; 
 constant auto_refresh : std_logic_vector(4 downto 0) := "00101";
 constant auto_refresh_wait : std_logic_vector(4 downto 0) := "00110";
 constant active : std_logic_vector(4 downto 0) := "00111";
 constant active_wait : std_logic_vector(4 downto 0) := "01000";
 constant first_read_s : std_logic_vector(4 downto 0) := "01001";
 constant active2 : std_logic_vector(4 downto 0) := "01010";
 constant read_wait : std_logic_vector(4 downto 0) := "01011";
 constant first_write_s : std_logic_vector(4 downto 0) := "01100";
 constant write_burst : std_logic_vector(4 downto 0) := "10110";
 constant active2_wait : std_logic_vector(4 downto 0) := "01101";
 constant write_wait : std_logic_vector(4 downto 0) := "01110";
 constant initcount_200 : std_logic_vector(4 downto 0) := "01111";
 constant initcount_200_wait : std_logic_vector(4 downto 0) := "10000";
 constant dummy_read_cycles : std_logic_vector(4 downto 0) := "10001";
 constant dummy_active : std_logic_vector(4 downto 0) := "10010";
 constant dummy_active_wait : std_logic_vector(4 downto 0) := "10011";
 constant dummy_read_s : std_logic_vector(4 downto 0) := "10100";
 constant dummy_read_wait : std_logic_vector(4 downto 0) := "10101";
 constant read_burst : std_logic_vector(4 downto 0) := "10111";
 constant dummy_first_read : std_logic_vector(4 downto 0) := "11000";


	attribute syn_noprune : boolean;
	attribute syn_noprune of rtl : architecture is true;
	attribute IOB : string;
   attribute IOB of ddr_cs_reg: label is "TRUE"; 
   attribute IOB of ddr_ba_reg: label is "TRUE";
	attribute IOB of ddr_address_reg: label is "TRUE";
   attribute IOB of ddr_we_reg: label is "TRUE";
   attribute IOB of ddr_cas_reg: label is "TRUE";
   attribute IOB of ddr_ras_reg: label is "TRUE";

   -- Load Mode command cycle time = tMRD -1
   constant mrd_count_value : std_logic_vector(0 downto 0) := "1";
   -- Precharge command period = tRP -1
   constant rp_count_value : std_logic_vector(1 downto 0) := "11";
   -- Auto Refresh command period = tRFC -1 
   constant rfc_count_value : std_logic_vector(4 downto 0) := "10011";
   -- Active to Read/Write = tRCD -1
   constant rcd_count_value : std_logic_vector(1 downto 0) := "01";
   -- active to active same bank = tRC-1
   constant rc_count_value : std_logic_vector(3 downto 0) := "1111";
   -- active to precharge = tras - 1
   constant ras_count_value : std_logic_vector(3 downto 0) := "1011";
   -- read to precharge = (AL + BL/2 + tRTP - 2) = (0+ BL/2 + 2 - 2) -1
   constant rtp_count_value : std_logic_vector(1 downto 0) := "01";
   -- write to precharge = (WL + BL/2 + tWR ) = (3+BL/2+4) -1
   constant wtp_count_value : std_logic_vector(3 downto 0) := "1000";
   --  write to read command time = (WL + BL/2 + tWTR) - 1 = (3+BL/2 + 2) -1
   constant wr_to_rd_count_value : std_logic_vector(3 downto 0) := "0110";
   -- cas latency
   constant cas_latency_value : std_logic_vector(2 downto 0) := "011";
   -- use 60 = 11.52 which is less than 15.6(requirement) - 4.0 (4K burst) 
   -- to guarantee that I always meet the requirement; even if doing a 4K burst
   constant max_ref_count : std_logic_vector(7 downto 0) := "01100000";
   -- phy mode: 1- direct clocking, 0- local clocking
   constant phy_mode : std_logic := '1';

COMPONENT dm_gen
   PORT (
      clk                     : IN std_logic;   
      rst                     : IN std_logic;   
      addr_fifo               : IN std_logic_vector(26 DOWNTO 0);   
      cmd_fifo                : IN std_logic_vector(23 DOWNTO 0);   
      wrburst_cnt             : IN std_logic_vector(8 DOWNTO 0);   
      burst_n_dword           : IN std_logic;   
      first_write             : IN std_logic;         
      ctrl_Dm_En              : OUT std_logic_vector(15 DOWNTO 0);         
      pci_be                  : IN std_logic_vector(7 DOWNTO 0));   
END COMPONENT;
					 
   -- internal signals
   SIGNAL init_count               :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL init_memory              :  std_logic;   
   SIGNAL count_200_cycle          :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL ref_flag                 :  std_logic;   
   SIGNAL ref_flag_266             :  std_logic;   
   SIGNAL ref_flag_266_r           :  std_logic;   
   SIGNAL auto_ref                 :  std_logic;   
   SIGNAL state                    :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL state_2r                 :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL state_3r                 :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL state_4r                 :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL state_5r                 :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL state_6r                 :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL col_addr_2r              :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL col_addr_3r              :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL col_addr_4r              :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL col_addr_5r              :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL col_addr_6r              :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL ddr_address_1r           :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL ddr_ba_1r                :  std_logic_vector(1 DOWNTO 0);   
   -- counters for ddr controller
   SIGNAL mrd_count                :  std_logic_vector(0 downto 0);   
   SIGNAL rp_count                 :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL rfc_count                :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL rcd_count                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL rc_count                 :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL refi_count               :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL wr_to_rd_count           :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL rd_to_wr_count           :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL cas_count                :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL cas_check_count          :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL ras_count                :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL rtp_count                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL wtp_count                :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL wrburst_cnt              :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL ctrl_WrEn_cnt            :  std_logic_vector(8 DOWNTO 0);   
   SIGNAL rdburst_cnt              :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL rd_cmd_cnt               :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL bank_addr_2r             :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_ras_2r               :  std_logic;   
   SIGNAL ddr_cas_2r               :  std_logic;   
   SIGNAL ddr_we_2r                :  std_logic;   
   SIGNAL ddr_ras_r                :  std_logic;   
   SIGNAL ddr_cas_r                :  std_logic;   
   SIGNAL ddr_we_r                 :  std_logic;   
   SIGNAL ddr_ras_3r               :  std_logic;   
   SIGNAL ddr_cas_3r               :  std_logic;   
   SIGNAL ddr_we_3r                :  std_logic;   
   SIGNAL rd_cmd_issued            :  std_logic;   
   SIGNAL wr_cmd_issued            :  std_logic;   
   SIGNAL WR                       :  std_logic;   
   SIGNAL RD                       :  std_logic;   
   SIGNAL ctrl_Dummyread_Start_1r  :  std_logic;   
   SIGNAL ctrl_Dummyread_Start_2r  :  std_logic;   
   SIGNAL ctrl_Dummyread_Start_3r  :  std_logic;   
   SIGNAL ctrl_Dummyread_Start_4r  :  std_logic;   
   -- FIFO read enable signals
   -- Rst and Enable signals for DQS logic
   -- Read and Write Enable signals to the phy interface
   SIGNAL dummy_read_en            :  std_logic;   
   SIGNAL count_200cycle_done      :  std_logic;   
   SIGNAL init_done                :  std_logic;   
   SIGNAL burst_cnt                :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL wdf_rcd_check            :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL countdown                :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL ddr_address_2r           :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL ddr_address_3r           :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL ddr_address_4r           :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL ddr_ba_2r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_ba_3r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_ba_4r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_ras_4r               :  std_logic;   
   SIGNAL ddr_cas_4r               :  std_logic;   
   SIGNAL ddr_we_4r                :  std_logic;   
   SIGNAL ddr_cs_1r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_cs_2r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_cs_3r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ddr_cs_4r                :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL read_or_write            :  std_logic;   
   SIGNAL valid_command            :  std_logic;   
   SIGNAL burst_n_dword            :  std_logic;   
   SIGNAL end_addr                 :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL calc_burst               :  std_logic_vector(22 DOWNTO 0);   
   SIGNAL first_write              :  std_logic;   
   SIGNAL burst_count_16B          :  std_logic_vector(6 DOWNTO 0);   -- how many 16B transfers
   SIGNAL end_addr_16B             :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL trunc_end_addr_16B       :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL trunc_start_addr_16B     :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL same_row_bank            :  std_logic;   
   SIGNAL write_n_read             :  std_logic;   
   SIGNAL end_addr_r               :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL valid_command_r          :  std_logic;   
   SIGNAL vld_cmd_hand             :  std_logic;   -- handshake signal for valid command
   SIGNAL early_act_count          :  std_logic_vector(11 DOWNTO 0);   -- early active command counter
   SIGNAL ctrl_RdEn_w              :  std_logic;   
   SIGNAL ctrl_RdEn_r2             :  std_logic;   
   SIGNAL ctrl_RdEn_r1             :  std_logic;   
   SIGNAL burst_zero               :  std_logic;   
   SIGNAL burst_zero_r             :  std_logic; 
	SIGNAL burst_half_i             :  std_logic;          
   --following logic used for corner case read/write of 0 bytes 
   SIGNAL act1                     :  std_logic;   
   SIGNAL act2                     :  std_logic;   
   -- commands to the memory
   SIGNAL cas_cond                 :  std_logic;   
   SIGNAL cas_cond_r1              :  std_logic;   
   SIGNAL cas_cond_r2              :  std_logic;   
   SIGNAL cas_cond_r3              :  std_logic;   
   SIGNAL cas_cond_r4              :  std_logic;   
   SIGNAL cas_cond_r5              :  std_logic;   
   SIGNAL read_cond                :  std_logic;   
   SIGNAL we_write_cond            :  std_logic;   
   SIGNAL we_write_cond_r1         :  std_logic;   
   SIGNAL we_write_cond_r2         :  std_logic;   
   SIGNAL we_write_cond_r3         :  std_logic;   
   SIGNAL we_write_cond_r4         :  std_logic;   
   SIGNAL we_write_cond_r5         :  std_logic;   
   -- register address outputs
   SIGNAL active_row_addr          :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL active2_row_addr         :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL active_bank_addr         :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL active2_bank_addr        :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL current_addr_r           :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL current_cs               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL cs_addr_2r               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL cs_addr_3r               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL cs_addr_4r               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL cs_addr_5r               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL cs_addr_6r               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL bank_addr_3r             :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL bank_addr_4r             :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL bank_addr_5r             :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL bank_addr_6r             :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ctrl_Dummyread_Start_reg      :  std_logic;     
   SIGNAL ctrl_Dqs_Rst_reg       :  std_logic;   
   SIGNAL ctrl_Dqs_En_reg        :  std_logic;     
   SIGNAL ctrl_WrEn_reg          :  std_logic;        
   SIGNAL not_ready_reg          :  std_logic;  
   SIGNAL dummy_read             :  std_logic;
	--synthesis translate_off
	signal fsm_monitor  : string(1 to 20);
	--synthesis translate_on

BEGIN
   ctrl_Dummyread_Start <= ctrl_Dummyread_Start_reg;
   ctrl_Dqs_Rst <= ctrl_Dqs_Rst_reg;
   ctrl_Dqs_En <= ctrl_Dqs_En_reg;
   ctrl_WrEn <= ctrl_WrEn_reg;
   ctrl_RdEn <= ctrl_RdEn_r1;
   ctrl_init_done <= init_done;
	not_ready <= not_ready_reg;
	burst_half_i <= burst_half;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (PCI_HIT_EARLY = '1') THEN
            write_n_read <= PCI_CBEA_EARLY(0);    
         ELSE
            write_n_read <= write_n_read;    
         END IF;
      END IF;
   END PROCESS;
   first_read <= '1' WHEN (state_4r = first_read_s) ELSE '0';
   same_row_bank <= '1' WHEN (addr_fifo(25 DOWNTO 12) = end_addr(25 DOWNTO 12)) ELSE '0';
   first_write <= '1' WHEN (state = first_write_s) ELSE '0';

   --following logic for generating S_Retry
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            not_ready_reg <= '0';    
         ELSE
            IF (((PCI_HIT_EARLY = '1') OR (PCI_HIT ='1')) AND (state /= 
            idle_ctrl) AND (NOT ((state = active_wait) AND (ras_count > "0100")))) THEN
               not_ready_reg <= '1';    
            ELSE
               IF ((state_3r = idle_ctrl) OR (S_DONE = '1')) 
               THEN
                  not_ready_reg <= '0';    
               ELSE
                  not_ready_reg <= not_ready_reg;    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;
   
   dm_gen_inst : dm_gen 
      PORT MAP (
         clk => clk266,
         rst => rst,
         addr_fifo => addr_fifo(26 DOWNTO 0),
         cmd_fifo => cmd_fifo(23 DOWNTO 0),
         wrburst_cnt => wrburst_cnt(8 DOWNTO 0),
         burst_n_dword => burst_n_dword,
         first_write => first_write,
         ctrl_Dm_En => ctrl_Dm_En(15 DOWNTO 0),
         pci_be => PCI_BE);   
   
   --fifo control signals
   ctrl_af_RdEn <= valid_command ;
   ctrl_cmd_RdEn <= valid_command ;
   WR <= '1' WHEN (cmd_fifo(20) = '1') ELSE '0';
   RD <= '1' WHEN (cmd_fifo(20) = '0') ELSE '0';
   burst_n_dword <= cmd_fifo(23) ;
   end_addr(26 DOWNTO 0) <= end_addr_r(26 DOWNTO 0) ;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            end_addr_r(26 DOWNTO 0) <= "000000000000000000000000000";    
         ELSE
            IF (cmd_fifo(19 DOWNTO 8) = "000000000000") THEN
               end_addr_r(26 DOWNTO 0) <= addr_fifo(26 DOWNTO 0);    
            ELSE
               end_addr_r(26 DOWNTO 0) <= addr_fifo(26 DOWNTO 0) + 
               cmd_fifo(19 DOWNTO 8) - "000000000000000000000000001";    
            END IF;
         END IF;
      END IF;
   END PROCESS;
   valid_command <= PCI_HIT ;

   PROCESS (clk266, rst)
   BEGIN
      IF (rst = '1') THEN
         valid_command_r <= '0';    
      ELSIF (clk266'EVENT AND clk266 = '1') THEN
         valid_command_r <= valid_command;    
      END IF;
   END PROCESS;

   PROCESS (clk266, rst)
   BEGIN
      IF (rst = '1') THEN
         vld_cmd_hand <= '0';    
      ELSIF (clk266'EVENT AND clk266 = '1') THEN
         IF (valid_command = '1') THEN
            vld_cmd_hand <= '1';    
         ELSE
            IF (state = first_write_s OR state = first_read_s OR state = precharge) 
            THEN
               vld_cmd_hand <= '0';    
            ELSE
               vld_cmd_hand <= vld_cmd_hand;    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            early_act_count(11 DOWNTO 0) <= X"000";    
         ELSE
            IF (PCI_HIT_EARLY = '1') THEN
               early_act_count(11 DOWNTO 0) <= X"10F";    
            ELSE
               IF (early_act_count(11 DOWNTO 0) /= X"000") THEN
                  early_act_count(11 DOWNTO 0) <= early_act_count(11 
                  DOWNTO 0) - X"001";    
               ELSE
                  early_act_count(11 DOWNTO 0) <= early_act_count(11 
                  DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   --to initialize memory
   
   PROCESS (clk266, rst)
   BEGIN
      IF (rst = '1') THEN
         init_memory <= '1';    
      ELSIF (clk266'EVENT AND clk266 = '1') THEN
         IF (init_count = "1010") THEN
            init_memory <= '0';    
         ELSE
            init_memory <= init_memory;    
         END IF;
      END IF;
   END PROCESS;

   -- mrd count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            mrd_count <= "0";    
         ELSE
            IF (state = load_mode_reg) THEN
               mrd_count <= "1";    
            ELSE
               IF (mrd_count /= "0") THEN
                  mrd_count <= mrd_count - mrd_count_value;    
               ELSE
                  mrd_count <= mrd_count;    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- rp count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rp_count(1 DOWNTO 0) <= "00";    
         ELSE
            IF (state = precharge) THEN
               rp_count(1 DOWNTO 0) <= rp_count_value;    
            ELSE
               IF (rp_count(1 DOWNTO 0) /= "00") THEN
                  rp_count(1 DOWNTO 0) <= rp_count(1 DOWNTO 0) - "01";   
               ELSE
                  rp_count(1 DOWNTO 0) <= rp_count(1 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- rfc count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rfc_count(4 DOWNTO 0) <= "00000";    
         ELSE
            IF (state = auto_refresh) THEN
               rfc_count(4 DOWNTO 0) <= rfc_count_value;    
            ELSE
               IF (rfc_count(4 DOWNTO 0) /= "00000") THEN
                  rfc_count(4 DOWNTO 0) <= rfc_count(4 DOWNTO 0) - 
                  "00001";    
               ELSE
                  rfc_count(4 DOWNTO 0) <= rfc_count(4 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- rcd count - 20ns
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rcd_count(1 DOWNTO 0) <= "00";    
         ELSE
            IF (((state = active) OR (state = active2) OR 
            (state = dummy_active)) AND (write_n_read = '1'))THEN
               rcd_count(1 DOWNTO 0) <= rcd_count_value + "10";    
            ELSE
               IF (((state = active) OR (state = active2) OR 
               (state = dummy_active)) AND (NOT (write_n_read = '1'))) THEN
                  rcd_count(1 DOWNTO 0) <= rcd_count_value;    
               ELSE
                  IF (rcd_count(1 DOWNTO 0) /= "00") THEN
                     rcd_count(1 DOWNTO 0) <= rcd_count(1 DOWNTO 0) - 
                     "01";    
                  ELSE
                     rcd_count(1 DOWNTO 0) <= rcd_count(1 DOWNTO 0);    
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- rc count - active to active on same bank
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rc_count(3 DOWNTO 0) <= "0000";    
         ELSE
            IF (state = active OR state = active2) THEN
               rc_count(3 DOWNTO 0) <= rc_count_value;    
            ELSE
               IF (rc_count(3 DOWNTO 0) /= "0000") THEN
                  rc_count(3 DOWNTO 0) <= rc_count(3 DOWNTO 0) - "0001"; 
               ELSE
                  rc_count(3 DOWNTO 0) <= rc_count(3 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- ras count - active to precharge
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ras_count(3 DOWNTO 0) <= "0000";    
         ELSE
            IF (state = active OR state = active2) THEN
               ras_count(3 DOWNTO 0) <= ras_count_value;    
            ELSE
               IF (ras_count(3 DOWNTO 0) /= "0000") THEN
                  ras_count(3 DOWNTO 0) <= ras_count(3 DOWNTO 0) - 
                  "0001";    
               ELSE
                  ras_count(3 DOWNTO 0) <= ras_count(3 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- rtp count - read to precharge
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rtp_count(1 DOWNTO 0) <= "00";    
         ELSE
            IF (state = first_read_s) THEN
               rtp_count(1 DOWNTO 0) <= rtp_count_value;    
            ELSE
               IF (rtp_count(1 DOWNTO 0) /= "00") THEN
                  rtp_count(1 DOWNTO 0) <= rtp_count(1 DOWNTO 0) - "01"; 
               ELSE
                  rtp_count(1 DOWNTO 0) <= rtp_count(1 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- wtp count - write to precharge
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            wtp_count(3 DOWNTO 0) <= "0000";    
         ELSE
            IF (state = first_write_s OR state = write_burst) THEN
               wtp_count(3 DOWNTO 0) <= wtp_count_value;    
            ELSE
               IF (wtp_count(3 DOWNTO 0) /= "0000") THEN
                  wtp_count(3 DOWNTO 0) <= wtp_count(3 DOWNTO 0) - 
                  "0001";    
               ELSE
                  wtp_count(3 DOWNTO 0) <= wtp_count(3 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- write to read counter 
   
   -- write to read includes : write latency + burst time + tWTR
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            wr_to_rd_count(3 DOWNTO 0) <= "0000";    
         ELSE
            IF (state = first_write_s) THEN
               wr_to_rd_count(3 DOWNTO 0) <= wr_to_rd_count_value;    
            ELSE
               IF (wr_to_rd_count(3 DOWNTO 0) /= "0000") THEN
                  wr_to_rd_count(3 DOWNTO 0) <= wr_to_rd_count(3 DOWNTO 
                  0) - "0001";    
               ELSE
                  wr_to_rd_count(3 DOWNTO 0) <= wr_to_rd_count(3 DOWNTO 
                  0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- read to write counter
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rd_to_wr_count(2 DOWNTO 0) <= "000";    
         ELSE
            IF (state = first_read_s) THEN
               rd_to_wr_count(2 DOWNTO 0) <= cas_latency_value;    
            ELSE
               IF (rd_to_wr_count(2 DOWNTO 0) /= "000") THEN
                  rd_to_wr_count(2 DOWNTO 0) <= rd_to_wr_count(2 DOWNTO 
                  0) - "001";    
               ELSE
                  rd_to_wr_count(2 DOWNTO 0) <= "000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- auto refresh interval counter in refresh_clk domain
   
   PROCESS (refresh_clk)
   BEGIN
      IF (refresh_clk'EVENT AND refresh_clk = '1') THEN
         IF (rst = '1') THEN
            refi_count <= "00000000";    
         ELSE
            IF (refi_count = max_ref_count) THEN
               refi_count <= "00000000";    
            ELSE
               refi_count <= refi_count + "00000001";    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   ref_flag <= '1' WHEN (refi_count = max_ref_count) ELSE '0';

   -- refresh detect in 266 MHz clock
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ref_flag_266 <= '0';    
            ref_flag_266_r <= '0';    
         ELSE
            ref_flag_266 <= ref_flag;    
         END IF;
         ref_flag_266_r <= ref_flag_266;    
      END IF;
   END PROCESS;

   --refresh flag detect  
   --auto_ref high indicates auto_refresh requirement   
   --auto_ref is held high until auto refresh command is issued.
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            auto_ref <= '0';    
         ELSE
            IF (ref_flag_266 = '1' AND ref_flag_266_r = '0') THEN
               auto_ref <= '1';    
            ELSE
               IF (state = auto_refresh) THEN
                  auto_ref <= '0';    
               ELSE
                  auto_ref <= auto_ref;    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- 200 clocks counter - count value : C8
   
   -- required for initialization
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            count_200_cycle(7 DOWNTO 0) <= "00000000";    
         ELSE
            IF (state = initcount_200) THEN
               count_200_cycle(7 DOWNTO 0) <= "11001000";    
            ELSE
               IF (count_200_cycle(7 DOWNTO 0) /= "00000000") THEN
                  count_200_cycle(7 DOWNTO 0) <= count_200_cycle(7 
                  DOWNTO 0) - "00000001";    
               ELSE
                  count_200_cycle(7 DOWNTO 0) <= "00000000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   count_200cycle_done <= '1' WHEN (init_memory = '1' AND count_200_cycle = "00000000") ELSE '0';

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            init_done <= '0';    
         ELSE
            IF (((init_count = "1001") AND (count_200cycle_done = '1')) 
            AND (phy_mode = '0')) THEN
               init_done <= '1';    
            ELSE
               IF ((phy_mode = '1') AND (phy_Dly_Slct_Done = '1')) THEN
                  init_done <= '1';    
               ELSE
                  init_done <= init_done;    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            init_count(3 DOWNTO 0) <= "0000";    
         ELSE
            IF (init_memory = '1') THEN
               IF (state = load_mode_reg OR state = precharge OR state = auto_refresh 
               OR state = dummy_read_cycles OR state = initcount_200) THEN
                  init_count(3 DOWNTO 0) <= init_count(3 DOWNTO 0) + 
                  "0001";    
               ELSE
                  init_count(3 DOWNTO 0) <= init_count(3 DOWNTO 0);    
               END IF;
            ELSE
               init_count(3 DOWNTO 0) <= "0000";    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- write burst count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            wrburst_cnt(8 DOWNTO 0) <= "000000000";    
         ELSE
            IF (state = first_write_s AND burst_n_dword = '0') THEN
               wrburst_cnt(8 DOWNTO 0) <= "000000010";    -- burst_cnt[2:0]};
            ELSE
               IF (state = first_write_s AND burst_n_dword = '1') THEN
                  wrburst_cnt(8 DOWNTO 0) <= calc_burst(8 DOWNTO 0);    
               ELSE
                  IF (wrburst_cnt(8 DOWNTO 0) /= "000000000") THEN
                     wrburst_cnt(8 DOWNTO 0) <= wrburst_cnt(8 DOWNTO 0) 
                     - "000000001";    
                  ELSE
                     wrburst_cnt(8 DOWNTO 0) <= "000000000";    
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (end_addr(26 DOWNTO 5), addr_fifo(26 DOWNTO 5))
   BEGIN
      calc_burst(22 DOWNTO 0) <= (end_addr(26 DOWNTO 5) - 
		                           addr_fifo(26 DOWNTO 5) + 
											"0000000000000000000001") & '0';    
   END PROCESS;

   -- count to generate write enable to the data path
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_WrEn_cnt(8 DOWNTO 0) <= "000000000";    
         ELSE
            ctrl_WrEn_cnt(8 DOWNTO 0) <= wrburst_cnt(8 DOWNTO 0);    
         END IF;
      END IF;
   END PROCESS;

   --write enable to data path
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_WrEn_reg <= '0';    
         ELSE
            IF ((ctrl_WrEn_cnt(8 DOWNTO 0) /= "000000000") AND (WR = '1')
            ) THEN
               ctrl_WrEn_reg <= '1';    
            ELSE
               ctrl_WrEn_reg <= '0';    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- DQS enable to data path
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_Dqs_En_reg <= '0';    
         ELSE
            IF ((state = first_write_s) OR ((wrburst_cnt /= "000000000") AND 
            (WR = '1'))) THEN
               ctrl_Dqs_En_reg <= '1';    
            ELSE
               ctrl_Dqs_En_reg <= '0';    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- DQS reset to data path
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_Dqs_Rst_reg <= '0';    
         ELSE
            IF (state = first_write_s) THEN
               ctrl_Dqs_Rst_reg <= '1';    
            ELSE
               ctrl_Dqs_Rst_reg <= '0';    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- cas count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            cas_count(2 DOWNTO 0) <= "000";    
         ELSE
            IF (state = first_read_s OR state = dummy_first_read) THEN
               cas_count(2 DOWNTO 0) <= "011";    
            ELSE
               IF (cas_count(2 DOWNTO 0) /= "000") THEN
                  cas_count(2 DOWNTO 0) <= cas_count(2 DOWNTO 0) - 
                  "001";    
               ELSE
                  cas_count(2 DOWNTO 0) <= "000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   --dummy_read enable
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            dummy_read_en <= '0';    
         ELSE
            IF (state = dummy_read_s) THEN
               dummy_read_en <= '1';    
            ELSE
               IF (phy_Dly_Slct_Done = '1') THEN
                  dummy_read_en <= '0';    
               ELSE
                  dummy_read_en <= dummy_read_en;    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- ctrl_Dummyread_Start signal generation to the data path module
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_Dummyread_Start_1r <= '0';    
         ELSE
            IF ((dummy_read_en = '1') AND (cas_count = "000")) THEN
               ctrl_Dummyread_Start_1r <= '1';    
            ELSE
               IF (phy_Dly_Slct_Done = '1') THEN
                  ctrl_Dummyread_Start_1r <= '0';    
               ELSE
                  ctrl_Dummyread_Start_1r <= ctrl_Dummyread_Start_1r;    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- register ctrl_Dummyread_Start signal
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_Dummyread_Start_2r <= '0';    
            ctrl_Dummyread_Start_3r <= '0';    
            ctrl_Dummyread_Start_4r <= '0';    
            ctrl_Dummyread_Start_reg <= '0';    
         ELSE
            ctrl_Dummyread_Start_2r <= ctrl_Dummyread_Start_1r;    
         END IF;
         ctrl_Dummyread_Start_3r <= ctrl_Dummyread_Start_2r;    
         ctrl_Dummyread_Start_4r <= ctrl_Dummyread_Start_3r;    
         ctrl_Dummyread_Start_reg <= ctrl_Dummyread_Start_4r;    
      END IF;
   END PROCESS;


   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            dummy_read <= '0';    
         ELSE
            IF (state = dummy_read_cycles or state = dummy_active or state = dummy_active_wait or 
                state = dummy_read_s or state = dummy_read_wait or state = dummy_first_read) THEN
               dummy_read <= '1';    
            ELSE
               dummy_read <= '0';      
            END IF;
         END IF;
      END IF;
   END PROCESS;


   -- read command count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rd_cmd_cnt(2 DOWNTO 0) <= "000";    
         ELSE
            IF (state = first_read_s OR state = read_burst) THEN
               rd_cmd_cnt(2 DOWNTO 0) <= "010";    
            ELSE
               IF (rd_cmd_cnt(2 DOWNTO 0) /= "000") THEN
                  rd_cmd_cnt(2 DOWNTO 0) <= rd_cmd_cnt(2 DOWNTO 0) - 
                  "001";    
               ELSE
                  rd_cmd_cnt(2 DOWNTO 0) <= "000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- cas-count wait for generating read enable
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            cas_check_count(2 DOWNTO 0) <= "000";    
         ELSE
            IF (state_4r = first_read_s) THEN
               -- in order to get the enable in the preamble              
               cas_check_count(2 DOWNTO 0) <= cas_latency_value - "001";    
            ELSE
               IF (cas_check_count(2 DOWNTO 0) /= "000") THEN
                  cas_check_count(2 DOWNTO 0) <= cas_check_count(2 
                  DOWNTO 0) - "001";    
               ELSE
                  cas_check_count(2 DOWNTO 0) <= "000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- read burst count
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            rdburst_cnt(7 DOWNTO 0) <= "00000000";    
         ELSE
            IF (state = first_read_s) THEN
               rdburst_cnt(7 DOWNTO 0) <= calc_burst(8 DOWNTO 1);    
            ELSE
               IF (rdburst_cnt(7 DOWNTO 0) /= "00000000" AND (state = 
               first_read_s OR state = read_burst)) THEN
                  rdburst_cnt(7 DOWNTO 0) <= rdburst_cnt(7 DOWNTO 0) - 
                  "00000001";    
               ELSE
                  rdburst_cnt(7 DOWNTO 0) <= rdburst_cnt(7 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;
  	   
		
   --rd_cmd_cnt will work for this particular design 
	--for a different frequency ratio or burst length
   --a separate counter may be needed 
   ctrl_RdEn_w <= '1' WHEN (rd_cmd_cnt /= "000" AND RD = '1') ELSE '0';

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ctrl_RdEn_r1 <= '0';    
            ctrl_RdEn_r2 <= '0';    
         ELSE
            ctrl_RdEn_r1 <= ctrl_RdEn_w;    
            ctrl_RdEn_r2 <= ctrl_RdEn_r1;    
         END IF;
      END IF;
   END PROCESS;

   wdf_rcd_check(1 DOWNTO 0) <= rcd_count_value - "01";
   -- write data fifo read enable
   end_addr_16B(26 DOWNTO 0) <= end_addr(26 DOWNTO 0) - 
   "000000000000000000000000011" ;
   trunc_end_addr_16B(6 DOWNTO 0) <= end_addr_16B(10 DOWNTO 4) ;
   trunc_start_addr_16B(6 DOWNTO 0) <= addr_fifo(10 DOWNTO 4) ;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            burst_count_16B(6 DOWNTO 0) <= "0000000";    
         ELSE
            IF (state_2r = first_write_s) THEN
               burst_count_16B(6 DOWNTO 0) <= trunc_end_addr_16B - 
               trunc_start_addr_16B + "0000001";    
            ELSE
               IF (burst_count_16B /= "0000000") THEN
                  burst_count_16B <= burst_count_16B - "0000001";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   burst_zero <= '0' WHEN (burst_count_16B = "0000000") ELSE '1';

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            burst_zero_r <= '0';    
         ELSE
            burst_zero_r <= burst_zero;    
         END IF;
      END IF;
   END PROCESS;

   ctrl_Wdf_RdEn <= burst_zero WHEN (addr_fifo(4) = '0') ELSE burst_zero_r;

   -- main control state machine  
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            state <= idle_ctrl;    
            countdown <= "11111";    
         ELSE
            CASE state IS
               WHEN idle_ctrl =>
                        IF (init_memory = '1') THEN
                           CASE init_count IS
                              WHEN "0000" =>
                                       state <= initcount_200;    
                              WHEN "0001" =>
                                       IF (count_200cycle_done = '1') 
                                       THEN
                                          state <= precharge;    
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "0010" =>
                                       IF (mrd_count = "0") THEN
                                          state <= load_mode_reg;    -- emr;
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "0011" =>
                                       IF (mrd_count = "0") THEN
                                          state <= load_mode_reg;    -- lmr;
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "0100" =>
                                       IF (mrd_count = "0") THEN
                                          state <= precharge;    
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "0101" =>
                                       IF (rp_count = "00") THEN
                                          state <= auto_refresh;    
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "0110" =>
                                       IF (rfc_count = "00000") THEN
                                          state <= auto_refresh;    
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "0111" =>
                                       IF (rfc_count = "00000") THEN
                                          state <= load_mode_reg;    
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "1000" =>
                                       IF (mrd_count = "0") THEN
                                          state <= initcount_200;    
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "1001" =>
                                       IF (count_200cycle_done = '1') 
                                       THEN
                                          IF (true) THEN
                                             -- direct clocking method requires dummy reads to train the 
                                             
                                             -- incoming DQS to the system clock
                                             
                                             state <= dummy_read_cycles;    
                                          ELSE
                                             state <= idle_ctrl;    
                                          END IF;
                                       ELSE
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN "1010" =>
                                       -- phy_Dly_Slct_Done is sent by the data path module
                                       
                                       -- after the delay has been selected
                                       
                                       IF (phy_Dly_Slct_Done = '1') 
                                       THEN
                                          state <= idle_ctrl;    
                                       END IF;
                              WHEN OTHERS =>
                                       NULL;
                              
                           END CASE;
                        ELSE
                           IF (auto_ref = '1') THEN
                              state <= auto_refresh;    
                           ELSE
                              IF ((PCI_HIT_EARLY AND NOT 
                              not_ready_reg) = '1') THEN
                                 state <= active;    
                              END IF;
                           END IF;
                        END IF;
               WHEN initcount_200 =>
                        state <= initcount_200_wait;    
               WHEN initcount_200_wait =>
                        IF (count_200_cycle = "11001000") THEN
                           state <= idle_ctrl;    
                        ELSE
                           state <= initcount_200_wait;    
                        END IF;
               WHEN dummy_read_cycles =>
                        state <= dummy_active;    
               WHEN dummy_active =>
                        state <= dummy_active_wait;    
               WHEN dummy_active_wait =>
                        IF (rcd_count = "00") THEN
                           state <= dummy_first_read;    
                        ELSE
                           state <= dummy_active_wait;    
                        END IF;
               WHEN dummy_first_read =>
                        state <= dummy_read_wait;    
               WHEN dummy_read_s =>
                        state <= dummy_read_wait;    
               WHEN dummy_read_wait =>
                        IF (phy_Dly_Slct_Done = '1') THEN
                           state <= precharge;    
                        ELSE
                           state <= dummy_read_s;    
                        END IF;
               WHEN precharge =>
                        state <= precharge_wait;    
               WHEN precharge_wait =>
                        IF (rp_count = "01") THEN
                           IF (auto_ref = '1') THEN
                              state <= auto_refresh;    
                           ELSE
                              state <= idle_ctrl;    
                           END IF;
                        ELSE
                           state <= precharge_wait;    
                        END IF;
               WHEN load_mode_reg =>
                        state <= mode_register_wait;    
               WHEN mode_register_wait =>
                        IF (mrd_count = "1") THEN
                           state <= idle_ctrl;    
                        ELSE
                           state <= mode_register_wait;    
                        END IF;
               WHEN auto_refresh =>
                        state <= auto_refresh_wait;    
               WHEN auto_refresh_wait =>
                        IF (rfc_count = "00001") THEN
                           state <= idle_ctrl;    
                        ELSE
                           state <= auto_refresh_wait;    
                        END IF;
               WHEN active =>
                        state <= active_wait;    
               WHEN active_wait =>
                        IF (early_act_count = X"000") THEN
                           state <= precharge;    
                        ELSE
                           IF (rcd_count > "00") THEN
                              state <= active_wait;    
                           ELSE
                              IF (((WR AND burst_half_i) AND vld_cmd_hand) 
                              = '1') THEN
                                 IF ((same_row_bank = '1') AND NOT wdf_empty = 
                                 '1') THEN
                                    state <= first_write_s;    
                                 ELSE
                                    IF (NOT same_row_bank = '1') THEN
                                       state <= active2;    
                                    END IF;
                                 END IF;
                              ELSE
                                 IF ((RD AND vld_cmd_hand) = '1') THEN
                                    state <= first_read_s;    
                                 ELSE
                                    state <= active_wait;    
                                 END IF;
                              END IF;
                           END IF;
                        END IF;
               WHEN active2 =>
                        state <= active2_wait;    
               WHEN active2_wait =>
                        IF ((rcd_count = "00" AND WR = '1') AND 
                        wdf_empty = '0') THEN
                           state <= first_write_s;    
                        ELSE
                           IF (rcd_count<="00" AND RD = '1') THEN
                              state <= read_burst;    
                           ELSE
                              state <= active2_wait;    
                           END IF;
                        END IF;
               WHEN first_write_s =>
                        state <= write_wait;    
               WHEN write_burst =>
                        state <= write_wait;    
               WHEN write_wait =>
                        IF ((burst_n_dword = '1') AND (wrburst_cnt >= 
                        "000000011")) THEN                
                           state <= write_burst;    
                        ELSE
                           IF (wtp_count = "0000") THEN
                              state <= precharge;    
                           ELSE
                              state <= write_wait;    
                           END IF;
                        END IF;
               WHEN first_read_s =>
                        IF (same_row_bank = '0') THEN
                           state <= active2;    
                        ELSE
                           state <= read_wait;    
                        END IF;
               WHEN read_burst =>
                        state <= read_wait;    
               WHEN read_wait =>
                        IF (rd_cmd_cnt /= "000") THEN
                           state <= read_wait;    
                        ELSE
                           IF ((burst_n_dword = '1') AND (rdburst_cnt >= 
                           "00000010")) THEN
                              state <= read_burst;    
                           ELSE
                              IF (rtp_count = "00") THEN
                                 state <= precharge;    
                              END IF;
                           END IF;
                        END IF;
               WHEN OTHERS =>
                        NULL;
               
            END CASE;
         END IF;
      END IF;
   END PROCESS;

   --register command outputs
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            state_2r <= idle_ctrl;    
            state_3r <= idle_ctrl;    
            state_4r <= idle_ctrl;    
            state_5r <= idle_ctrl;    
            state_6r <= idle_ctrl;    
         ELSE
            state_2r <= state;    
            state_3r <= state_2r;    
            state_4r <= state_3r;    
            state_5r <= state_4r;    
            state_6r <= state_5r;    
         END IF;
      END IF;
   END PROCESS;
   act1 <= '1' WHEN ((state = idle_ctrl) AND (PCI_HIT_EARLY = '1') AND  
   (init_memory = '0')) ELSE '0';
   act2 <= '1' WHEN (((state = first_read_s) AND (same_row_bank = '0')) OR 
   ((state = active_wait) AND (WR ='1') AND 
   (rcd_count = "00") AND (burst_half_i = '1') AND (vld_cmd_hand = '1')
   AND (same_row_bank = '0'))) ELSE '0' ;

   -- commands to the memory
   
ddr_ras_reg:   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_ras_r <= '1';    
         ELSE
            IF ((state = load_mode_reg) OR 
				    (state = precharge) OR 
                (act1 = '1')      OR 
					 (act2 = '1')      OR 
					 (state = auto_refresh) OR 
                (state = dummy_active)) THEN
               ddr_ras_r <= '0';    
            ELSE
               ddr_ras_r <= '1';    
            END IF;
         END IF;
      END IF;
   END PROCESS;
   cas_cond <= '1' WHEN ((state = first_write_s) OR (state = write_burst)) ELSE '0' ;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            cas_cond_r1 <= '0';    
            cas_cond_r2 <= '0';    
            cas_cond_r3 <= '0';    
            cas_cond_r4 <= '0';    
            cas_cond_r5 <= '0';    
         ELSE
            cas_cond_r1 <= cas_cond;    
            cas_cond_r2 <= cas_cond_r1;    
            cas_cond_r3 <= cas_cond_r2;    
            cas_cond_r4 <= cas_cond_r3;    
            cas_cond_r5 <= cas_cond_r4;    
         END IF;
      END IF;
   END PROCESS;
   read_cond <= '1' WHEN(((state = active_wait) AND (RD = '1') AND (rcd_count <="00") AND (vld_cmd_hand = '1') AND (early_act_count /= X"000")) OR 
	              ((state = active2_wait) AND (rcd_count<="00") AND (RD = '1')) OR 
					  ((state = read_wait) AND (rd_cmd_cnt = "000") AND (burst_n_dword = '1') AND (rdburst_cnt >= "00000010"))) 
					  ELSE '0';

ddr_cas_reg:   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_cas_r <= '1';    
         ELSE
            IF ((state = load_mode_reg)   OR 
				    (cas_cond_r5 = '1') OR 
                (read_cond = '1')   OR 
					 (state = dummy_first_read)   OR 
                (state = auto_refresh)   OR
					 (state = dummy_read_s)) THEN
               ddr_cas_r <= '0';    
            ELSE
               ddr_cas_r <= '1';    
            END IF;
         END IF;
      END IF;
   END PROCESS;
   we_write_cond <= '1' WHEN ((state = first_write_s) OR (state = write_burst)) ELSE '0';

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            we_write_cond_r1 <= '0';    
            we_write_cond_r2 <= '0';    
            we_write_cond_r3 <= '0';    
            we_write_cond_r4 <= '0';    
            we_write_cond_r5 <= '0';    
         ELSE
            we_write_cond_r1 <= we_write_cond;    
            we_write_cond_r2 <= we_write_cond_r1;    
            we_write_cond_r3 <= we_write_cond_r2;    
            we_write_cond_r4 <= we_write_cond_r3;    
            we_write_cond_r5 <= we_write_cond_r4;    
         END IF;
      END IF;
   END PROCESS;

   -- commands to the memory
   
ddr_we_reg:   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_we_r <= '1';    
         ELSE
            IF ((state = load_mode_reg)       OR 
				    (we_write_cond_r5 = '1')OR 
                (state = precharge)) THEN
               ddr_we_r <= '0';    
            ELSE
               ddr_we_r <= '1';    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   --register commands to the memory
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_ras_2r <= '1';    
            ddr_cas_2r <= '1';    
            ddr_we_2r <= '1';    
         ELSE
            ddr_ras_2r <= ddr_ras_r; 
				ddr_cas_2r <= ddr_cas_r;    
            ddr_we_2r <= ddr_we_r;    
         END IF;  
      END IF;
   END PROCESS;

   --register commands to the memory
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_ras_3r <= '1';    
            ddr_cas_3r <= '1';    
            ddr_we_3r <= '1';    
         ELSE
            ddr_ras_3r <= ddr_ras_2r; 
				ddr_cas_3r <= ddr_cas_2r;    
            ddr_we_3r <= ddr_we_2r;      
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk266, rst)
   BEGIN
      IF (rst = '1') THEN
         active_row_addr(11 DOWNTO 0) <= "000000000000";    
         active_bank_addr(1 DOWNTO 0) <= "00";    
         active2_row_addr(11 DOWNTO 0) <= "000000000000";    
         active2_bank_addr(1 DOWNTO 0) <= "00";    
      ELSIF (clk266'EVENT AND clk266 = '1') THEN
         IF (valid_command_r = '1') THEN
            active_row_addr(11 DOWNTO 0) <= addr_fifo(25 DOWNTO 14);    
            active_bank_addr(1 DOWNTO 0) <= addr_fifo(13 DOWNTO 12);    
            active2_row_addr(11 DOWNTO 0) <= end_addr(25 DOWNTO 14);    
            active2_bank_addr(1 DOWNTO 0) <= end_addr(13 DOWNTO 12);    
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            current_addr_r(26 DOWNTO 0) <= 
            "000000000000000000000000000";    
         ELSE
            IF (state_2r = active) THEN
               --valid_command_r) begin 
               
               current_addr_r(26 DOWNTO 0) <= addr_fifo(26 DOWNTO 5) & "00000";    -- start on 32B boundary
            ELSE
               IF (burst_n_dword = '1' AND (state_2r = first_write_s OR 
               state_2r = write_burst OR state = first_read_s OR state = read_burst))
               THEN
                  --(state == `read_wait || state == `write_wait)) begin
                  
                  current_addr_r(26 DOWNTO 0) <= current_addr_r(26 
                  DOWNTO 0) + "000000000000000000000100000";    
               ELSE
                  current_addr_r(26 DOWNTO 0) <= current_addr_r(26 
                  DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            col_addr_2r(8 DOWNTO 0) <= "000000000";    
            col_addr_3r(8 DOWNTO 0) <= "000000000";    
            col_addr_4r(8 DOWNTO 0) <= "000000000";    
            col_addr_5r(8 DOWNTO 0) <= "000000000";    
            col_addr_6r(8 DOWNTO 0) <= "000000000";    
         ELSE
            col_addr_2r(8 DOWNTO 0) <= current_addr_r(11 DOWNTO 3);    
            col_addr_3r(8 DOWNTO 0) <= col_addr_2r(8 DOWNTO 0);    
            col_addr_4r(8 DOWNTO 0) <= col_addr_3r(8 DOWNTO 0);    
            col_addr_5r(8 DOWNTO 0) <= col_addr_4r(8 DOWNTO 0);    
            col_addr_6r(8 DOWNTO 0) <= col_addr_5r(8 DOWNTO 0);    
         END IF;
      END IF;
   END PROCESS;

ddr_address_reg:   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1' or dummy_read = '1') THEN
            ddr_address_1r <= "000000000000";    
         ELSE
            IF (init_memory = '1') THEN
               IF (state = precharge) THEN
                  ddr_address_1r <= "010000000000";    -- A10 = 1 for precharge all
               ELSE
                  IF (state = load_mode_reg AND init_count = "0011") THEN
                     ddr_address_1r <= "00010" & "110" & '0' & "010";    --  A8 == 1 for DLL reset
                  ELSE
                     IF (state = load_mode_reg AND init_count = "0111") THEN
                        ddr_address_1r <= "00000" & "110" & '0' & "010";    --  OPE CAS Burst_Type Burst_Length
                     ELSE
                        ddr_address_1r <= "000000000000";    
                     END IF;
                  END IF;
               END IF;
            ELSE
               IF (act2 = '1') THEN
                  ddr_address_1r <= active2_row_addr(11 DOWNTO 0);    
               ELSE
                  IF (read_cond = '1') THEN
                     ddr_address_1r <= "000" & current_addr_r(11 DOWNTO 
                     3);    
                  ELSE
                     IF (state_6r = first_write_s OR state_6r = write_burst) THEN
                        ddr_address_1r <= "000" & col_addr_6r(8 DOWNTO 0)
                        ;    
                     ELSE
                        IF (state = precharge) THEN
                           ddr_address_1r <= "010000000000";    
                        ELSE
                           ddr_address_1r <= PCI_ADDR_EARLY(25 DOWNTO 14)
                           ;    
                        END IF;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   current_cs <= "01" WHEN (current_addr_r(26) = '0') ELSE "10";

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            cs_addr_2r(1 DOWNTO 0) <= "00";    
            cs_addr_3r(1 DOWNTO 0) <= "00";    
            cs_addr_4r(1 DOWNTO 0) <= "00";    
            cs_addr_5r(1 DOWNTO 0) <= "00";    
            cs_addr_6r(1 DOWNTO 0) <= "00";    
         ELSE
            cs_addr_2r(1 DOWNTO 0) <= current_cs(1 DOWNTO 0);    
            cs_addr_3r(1 DOWNTO 0) <= cs_addr_2r(1 DOWNTO 0);    
            cs_addr_4r(1 DOWNTO 0) <= cs_addr_3r(1 DOWNTO 0);    
            cs_addr_5r(1 DOWNTO 0) <= cs_addr_4r(1 DOWNTO 0);    
            cs_addr_6r(1 DOWNTO 0) <= cs_addr_5r(1 DOWNTO 0);    
         END IF;
      END IF;
   END PROCESS;

ddr_cs_reg:   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_cs_1r(1 DOWNTO 0) <= "11";    
         ELSE
            IF ((read_cond = '1') OR 
				    (state = dummy_read_s) OR 
                (state = dummy_read_wait) OR 
					 (state = dummy_first_read)) THEN
               ddr_cs_1r(1 DOWNTO 0) <= current_cs(1 DOWNTO 0);    
            ELSE
               IF (state_6r = first_write_s OR state_6r = write_burst) THEN
                  ddr_cs_1r(1 DOWNTO 0) <= cs_addr_6r(1 DOWNTO 0);    
               ELSE
                  ddr_cs_1r(1 DOWNTO 0) <= "00";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            bank_addr_2r(1 DOWNTO 0) <= "00";    
            bank_addr_3r(1 DOWNTO 0) <= "00";    
            bank_addr_4r(1 DOWNTO 0) <= "00";    
            bank_addr_5r(1 DOWNTO 0) <= "00";    
            bank_addr_6r(1 DOWNTO 0) <= "00";    
         ELSE
            bank_addr_2r(1 DOWNTO 0) <= current_addr_r(13 DOWNTO 12);    
            bank_addr_3r(1 DOWNTO 0) <= bank_addr_2r(1 DOWNTO 0);    
            bank_addr_4r(1 DOWNTO 0) <= bank_addr_3r(1 DOWNTO 0);    
            bank_addr_5r(1 DOWNTO 0) <= bank_addr_4r(1 DOWNTO 0);    
            bank_addr_6r(1 DOWNTO 0) <= bank_addr_5r(1 DOWNTO 0);    
         END IF;
      END IF;
   END PROCESS;

ddr_ba_reg:   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1' or dummy_read = '1') THEN
            ddr_ba_1r(1 DOWNTO 0) <= "00";    
         ELSE
            IF (init_memory = '1' AND state = load_mode_reg) THEN
               IF (init_count = "0010") THEN
                  ddr_ba_1r(1 DOWNTO 0) <= "01";    -- emr2 2'b10 
               ELSE
                  ddr_ba_1r(1 DOWNTO 0) <= "00";    -- lmr
               END IF;
            ELSE
               IF (act2 = '1') THEN
                  ddr_ba_1r(1 DOWNTO 0) <= active2_bank_addr(1 DOWNTO 0);    
               ELSE
                  IF ((read_cond = '1') OR 
						    (state = first_read_s) OR 
                      (state = read_burst) OR 
                      (state = read_wait)) THEN
                     ddr_ba_1r(1 DOWNTO 0) <= current_addr_r(13 DOWNTO 12);    
                  ELSE
                     IF (state_6r = first_write_s OR state_6r = write_burst OR 
                     state_6r = write_wait) THEN
                        ddr_ba_1r(1 DOWNTO 0) <= bank_addr_6r(1 DOWNTO 0);    
                     ELSE
                        ddr_ba_1r(1 DOWNTO 0) <= PCI_ADDR_EARLY(13 
                        DOWNTO 12);    
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- need to register commands and address signals inside IOBs
   
   PROCESS (clk266)
   BEGIN
      IF (clk266'EVENT AND clk266 = '1') THEN
         IF (rst = '1') THEN
            ddr_address_2r(11 DOWNTO 0) <= "111111111111";    
            ddr_ba_2r(1 DOWNTO 0) <= "11";    
            ddr_cs_2r(1 DOWNTO 0) <= "11";    
            ddr_address_3r(11 DOWNTO 0) <= "111111111111";    
            ddr_ba_3r(1 DOWNTO 0) <= "11";    
            ddr_cs_3r(1 DOWNTO 0) <= "11";    
            ddr_address_4r(11 DOWNTO 0) <= "111111111111";    
            ddr_ba_4r(1 DOWNTO 0) <= "11";    
            ddr_cs_4r(1 DOWNTO 0) <= "11";    
            ddr_ras_4r <= '1';    
            ddr_cas_4r <= '1';    
            ddr_we_4r <= '1';    
         ELSE
            ddr_address_2r(11 DOWNTO 0) <= ddr_address_1r(11 DOWNTO 0);  
            ddr_ba_2r(1 DOWNTO 0) <= ddr_ba_1r(1 DOWNTO 0);    
            ddr_cs_2r(1 DOWNTO 0) <= ddr_cs_1r(1 DOWNTO 0);    
            ddr_address_3r(11 DOWNTO 0) <= ddr_address_2r(11 DOWNTO 0);  
            ddr_ba_3r(1 DOWNTO 0) <= ddr_ba_2r(1 DOWNTO 0);    
            ddr_cs_3r(1 DOWNTO 0) <= ddr_cs_2r(1 DOWNTO 0);    
            ddr_address_4r(11 DOWNTO 0) <= ddr_address_3r(11 DOWNTO 0);  
            ddr_ba_4r(1 DOWNTO 0) <= ddr_ba_3r(1 DOWNTO 0);    
            ddr_cs_4r(1 DOWNTO 0) <= ddr_cs_3r(1 DOWNTO 0);    
            ddr_ras_4r <= ddr_ras_3r;    
            ddr_cas_4r <= ddr_cas_3r;    
            ddr_we_4r <= ddr_we_3r;    
         END IF;
      END IF;
   END PROCESS;
   ddr_odt <= '0' ;
   ddr_cke <= '1' ;
   ddr_cs_L <= ddr_cs_1r ;
   ddr_address(11 DOWNTO 0) <= ddr_address_1r(11 DOWNTO 0) ;
   ddr_ba(1 DOWNTO 0) <= ddr_ba_1r(1 DOWNTO 0) ;
   ddr_ras_L <= ddr_ras_r ;
   ddr_cas_L <= ddr_cas_r ;
   ddr_we_L <= ddr_we_r ;
 --synthesis translate_off
   PROCESS (state)
   BEGIN
      CASE state IS
         WHEN "00000" =>
                  fsm_monitor <= "IDLE                ";     
         WHEN "00001" =>
                  fsm_monitor <= "LMR                 ";    
         WHEN "00010" =>
                  fsm_monitor <= "MRW                 ";    
         WHEN "00011" =>
                  fsm_monitor <= "PRE                 ";    
         WHEN "00100" =>
                  fsm_monitor <= "PRE_WAIT            ";    
         WHEN "00101" =>
                  fsm_monitor <= "AUTO_REF            ";    
         WHEN "00110" =>
                  fsm_monitor <= "AUTO_REF_WAIT       ";    
         WHEN "00111" =>
                  fsm_monitor <= "ACTIVE              ";    
         WHEN "01000" =>
                  fsm_monitor <= "ACTIVE_WAIT         ";    
         WHEN "01010" =>
                  fsm_monitor <= "ACTIVE2             ";    
         WHEN "01101" =>
                  fsm_monitor <= "ACTIVE2_WAIT        ";    
         WHEN first_read_s =>
                  fsm_monitor <= "FIRST_READ          ";    
         WHEN "01011" =>
                  fsm_monitor <= "READ_WAIT           ";    
         WHEN "01100" =>
                  fsm_monitor <= "FIRST_WRITE         ";    
         WHEN "01110" =>
                 fsm_monitor <=  "WRITE_WAIT          ";    
         WHEN "01111" =>
                  fsm_monitor <= "INIT_200            ";    
         WHEN initcount_200_wait =>
                  fsm_monitor <= "INIT_200_WAIT       ";    
         WHEN "10001" =>
                  fsm_monitor <= "DUMMY_READ          ";    
         WHEN "10010" =>
                  fsm_monitor <= "DUMMY_ACTIVE        ";    
         WHEN "10011" =>
                  fsm_monitor <= "DUMMY_ACTIVE_WAIT   ";    
         WHEN "10100" =>
                  fsm_monitor <= "DUMMY_READ          ";    
         WHEN "10101" =>
                  fsm_monitor <= "DUMMY_READ_WAIT     ";    
         WHEN write_burst =>
                  fsm_monitor <= "WRITE_BURST         ";    
         WHEN "10111" =>
                  fsm_monitor <= "READ_BURST          ";    
         WHEN dummy_first_read =>
                  fsm_monitor <= "DUMMY_FIRST_READ    ";      
         WHEN OTHERS  =>
                  fsm_monitor <= "ILLEGAL             ";    
         
      END CASE;
   END PROCESS;
--synthesis translate_on

END rtl;
